"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateDocs = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const execa_1 = require("execa");
const fs_extra_1 = __importDefault(require("fs-extra"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const json_to_pretty_yaml_1 = __importDefault(require("json-to-pretty-yaml"));
const store_1 = __importStar(require("./store"));
const requiredFields_1 = require("../utils/requiredFields");
const restCodes_1 = require("./restCodes");
function generateDocs() {
    return __awaiter(this, void 0, void 0, function* () {
        let jsonOpenApiPath = "";
        let config = store_1.getConfig();
        if (!process.env.DOC_MODE && !config.generateDoc) {
            return;
        }
        console.log("\nProject started in DOC_MODE. Documentation will now generate 📑\nProcess will kill itself at the end.\n");
        setTimeout(() => __awaiter(this, void 0, void 0, function* () {
            config = store_1.getConfig();
            // generate tmp folder
            const tmpPath = `${process.cwd()}/docs`;
            const openApiDoc = _initOpenApiDoc();
            yield fs_extra_1.default.ensureDir(tmpPath);
            const routes = store_1.getRoutes();
            const port = config.port || restCodes_1.DEFAULT_PORT;
            for (const route of routes) {
                const tag = _extractTag(route.path);
                const res = yield node_fetch_1.default(`http://localhost:${port}${route.path}`, {
                    method: route.method,
                    headers: config.doc && config.doc.headers ? config.doc.headers : [],
                    body: ["PUT", "POST"].includes(route.method)
                        ? JSON.stringify({})
                        : undefined,
                }).then((r) => r.json());
                let formatedTypeName = "";
                const params = [];
                if (route.path === "/") {
                    formatedTypeName = "Home";
                }
                else {
                    formatedTypeName = route.path
                        .split("/")
                        .map((word) => {
                        word.length ? word[0].toUpperCase() + word.substring(1) : "";
                        if (word.length) {
                            let wordFormated = word;
                            if (word.includes(":")) {
                                wordFormated = wordFormated.replace(":", "");
                                params.push(wordFormated);
                            }
                            return wordFormated[0].toUpperCase() + wordFormated.substring(1);
                        }
                        else {
                            return "";
                        }
                    })
                        .join("");
                }
                try {
                    const jsonFilePath = `${tmpPath}/tmp.json`;
                    yield fs_extra_1.default.writeJSON(jsonFilePath, res);
                    const typeName = `${route.method[0] +
                        route.method.substring(1).toLowerCase()}${formatedTypeName}Response`;
                    const cm1 = yield execa_1.command(`node ${process.cwd()}/node_modules/quicktype/dist/cli/index.js ${jsonFilePath} -l typescript --just-types -t ${typeName}`);
                    const typeFile = `${tmpPath}/tmp.ts`;
                    yield fs_extra_1.default.writeFile(typeFile, cm1.stdout);
                    // TODO use node path
                    const { stdout } = yield execa_1.command(`node ${process.cwd()}/node_modules/ts-to-openapi/dist/bin/ts-to-openapi -f ${typeFile} -t ${typeName}`);
                    let fixed = stdout.replace("/**", "");
                    fixed = fixed.replace("*/", "");
                    fixed = fixed.replace(/ \* /g, "");
                    fixed = fixed.replace("@swagger", "---");
                    const doc = js_yaml_1.default.safeLoad(fixed);
                    const keys = Object.keys(doc.components.schemas);
                    // @ts-ignore
                    keys.forEach((key) => {
                        // @ts-ignore
                        openApiDoc.components.schemas[key] = doc.components.schemas[key];
                    });
                    let pathToUse = route.path;
                    params.forEach((param) => {
                        pathToUse = pathToUse.replace(`:${param}`, `{${param}}`);
                    });
                    // @ts-ignore
                    if (!openApiDoc.paths[pathToUse]) {
                        // @ts-ignore
                        openApiDoc.paths[pathToUse] = {};
                    }
                    // @ts-ignore
                    openApiDoc.paths[pathToUse][route.method.toLowerCase()] = {
                        summary: route.summary,
                        parameters: generateParameters(params, route.fields),
                        description: route.description,
                        responses: {
                            "200": {
                                description: "",
                                content: {
                                    "application/json": {
                                        schema: {
                                            $ref: `#/components/schemas/${typeName}`,
                                        },
                                    },
                                },
                            },
                        },
                    };
                    if (tag) {
                        Object.assign(openApiDoc.paths[pathToUse][route.method.toLowerCase()], tag);
                    }
                    jsonOpenApiPath = `${tmpPath}/openApiDoc.json`;
                    yield fs_extra_1.default.writeJSON(jsonOpenApiPath, openApiDoc);
                    yield Promise.all([fs_extra_1.default.remove(typeFile), fs_extra_1.default.remove(jsonFilePath)]);
                }
                catch (e) {
                    console.log(e);
                }
            }
            const yamlOpenApiPath = yield generateYamlFromOpenApiJson(jsonOpenApiPath);
            yield execa_1.command(`node ${process.cwd()}/node_modules/openapi-markdown/bin/index.js -i ${yamlOpenApiPath}`);
            console.log("\nOpenAPI Documentation generated under /docs 📙\nJob's done - shuting down process.\n");
            process.exit();
        }), 1000);
    });
}
exports.generateDocs = generateDocs;
function _initOpenApiDoc() {
    var _a, _b, _c, _d, _e, _f;
    const config = store_1.default.get("config");
    const file = {
        openapi: "3.0.0",
        info: {
            version: (_b = (_a = config.doc) === null || _a === void 0 ? void 0 : _a.version) !== null && _b !== void 0 ? _b : "",
            title: (_d = (_c = config.doc) === null || _c === void 0 ? void 0 : _c.title) !== null && _d !== void 0 ? _d : "",
            description: (_f = (_e = config.doc) === null || _e === void 0 ? void 0 : _e.description) !== null && _f !== void 0 ? _f : "",
        },
        paths: {},
        components: {
            schemas: {},
        },
    };
    if (config.doc && config.doc.servers) {
        file["servers"] = config.doc.servers;
    }
    return file;
}
function generateYamlFromOpenApiJson(jsonFilePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const jsonData = yield fs_extra_1.default.readJSON(jsonFilePath);
        const yamlData = json_to_pretty_yaml_1.default.stringify(jsonData);
        const yamlFile = "docs/opanApiDoc.yaml";
        yield fs_extra_1.default.outputFile(yamlFile, yamlData);
        return yamlFile;
    });
}
function generateParameters(params = [], fields = []) {
    const parameters = [];
    params.forEach((param) => {
        parameters.push({
            name: param,
            in: "path",
            required: true,
            description: "",
            schema: {
                type: "string",
            },
        });
    });
    fields.forEach((param) => {
        const parameter = {
            name: typeof param === "string" ? param.replace("!", "") : param.name,
            in: "query",
            required: typeof param === "string"
                ? requiredFields_1.isFieldRequired(param)
                : typeof param.optional === "boolean"
                    ? param.optional
                    : false,
            description: typeof param === "string" ? "" : param.description || "",
            schema: {
                type: typeof param === "string"
                    ? "string"
                    : _getOpenApiType(param.type) || "string",
            },
        };
        if (typeof param === "object" && param.enum) {
            // @ts-ignore
            parameter.schema["enum"] = param.enum;
        }
        // @ts-ignore
        parameters.push(parameter);
    });
    return parameters;
}
function _getOpenApiType(type) {
    if (!type) {
        return "string";
    }
    if (type === "number") {
        return "integer";
    }
    return type;
}
function _extractTag(route) {
    // will remove the /
    let currentRoute = route.substring(1);
    if (!currentRoute.length) {
        return undefined;
    }
    return {
        tags: [
            currentRoute.substring(0, currentRoute.indexOf("/") === -1
                ? undefined
                : currentRoute.indexOf("/")),
        ],
    };
}
