"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExpressApp = exports.addRoute = exports.createServer = void 0;
const express_1 = __importDefault(require("express"));
const morgan_1 = __importDefault(require("morgan"));
const cors_1 = __importDefault(require("cors"));
const dotenv_1 = __importDefault(require("dotenv"));
const requiredFields_1 = __importStar(require("../utils/requiredFields"));
const store_1 = __importStar(require("./store"));
const errorHandler_1 = __importDefault(require("./errorHandler"));
const generator_1 = require("./generator");
const restCodes_1 = require("./restCodes");
function createServer(config = { useCors: true }) {
    require("express-async-errors");
    dotenv_1.default.config(config.dotenvConfig);
    const expressApp = express_1.default();
    if (config.uses) {
        config.uses.forEach((use) => {
            expressApp.use(use);
        });
    }
    store_1.default.set("expressApp", expressApp);
    store_1.default.set("config", config);
    store_1.updateRouteStore([]);
    expressApp.use(express_1.default.json());
    if (config.morgan) {
        expressApp.use(morgan_1.default(config.morgan.format, config.morgan.options));
    }
    else {
        expressApp.use(morgan_1.default("short"));
    }
    if (config.useCors) {
        expressApp.use(cors_1.default());
    }
    expressApp.set("port", (config === null || config === void 0 ? void 0 : config.port) || process.env.PORT || restCodes_1.DEFAULT_PORT);
    expressApp.listen(expressApp.get("port"), () => {
        console.log(config.readyMessage
            ? config.readyMessage
            : `Ready on http://localhost:${expressApp.get("port")}`);
    });
    // The error handler needs to be declared last.
    setTimeout(() => {
        expressApp.use(errorHandler_1.default);
    }, 100);
    generator_1.generateDocs();
    return expressApp;
}
exports.createServer = createServer;
function addRoute(cb, { method = "GET", path = "/", middlewares = [], secure, fields, summary = "", description = "", } = {
    method: "GET",
    path: "/",
    middlewares: [],
}) {
    const expressFn = _getExpressMethodFn(method);
    const config = store_1.default.get("config");
    // ---Update routes list to our system.
    const routes = store_1.getRoutes();
    routes.push({
        path,
        method,
        fields: fields || [],
        summary,
        description,
    });
    store_1.updateRouteStore(routes);
    // ----------
    let authMiddleware = config.auth && config.auth.secureAllRoutes
        ? config.auth.authMiddleware
        : _placeholder;
    if (secure &&
        (!config.auth || (config.auth && !config.auth.authMiddleware))) {
        throw new Error(`Cannot use config.secure on route ${path} as authMiddleware is undefined.`);
    }
    else if (typeof secure === "boolean" && !secure) {
        authMiddleware = _placeholder;
    }
    expressFn(path, authMiddleware, ...middlewares, (req, res) => __awaiter(this, void 0, void 0, function* () {
        if (fields && fields.length) {
            const requiredFields = [];
            fields.forEach((field) => {
                if (typeof fields === "string" && requiredFields_1.isFieldRequired(field)) {
                    requiredFields.push(field);
                }
                else if (typeof field === "object" && !field.optional) {
                    requiredFields.push(field.name);
                }
            });
            if (requiredFields.length) {
                requiredFields_1.default({
                    fields: requiredFields,
                    req,
                });
            }
        }
        let params = {};
        let ctx = {};
        const data = {};
        if (req.params) {
            params = req.params;
        }
        if (req.body) {
            Object.assign(data, req.body);
        }
        if (req.query) {
            Object.assign(data, req.query);
        }
        // @ts-ignore
        if (req.ctx) {
            // @ts-ignore
            ctx = req.ctx;
        }
        let result = { error: null };
        result = yield cb({ req, res, data, params, ctx });
        if (!result) {
            result = { error: null };
        }
        // TODO add support to support other rest code then 200 for instance
        res.json(result);
    }));
}
exports.addRoute = addRoute;
function getExpressApp() {
    const expressApp = store_1.default.get("expressApp");
    if (!expressApp) {
        throw new Error("expressApp not initialized.");
    }
    return expressApp;
}
exports.getExpressApp = getExpressApp;
function _getExpressMethodFn(method) {
    const expressApp = store_1.default.get("expressApp");
    switch (method) {
        case "GET":
            return expressApp.get.bind(expressApp);
        case "POST":
            return expressApp.post.bind(expressApp);
        case "DELETE":
            return expressApp.delete.bind(expressApp);
        case "PUT":
            return expressApp.put.bind(expressApp);
        case "PATCH":
            return expressApp.patch.bind(expressApp);
        default:
            return expressApp.get.bind(expressApp);
    }
}
function _placeholder(_, __, next) {
    next();
}
